@@ -107,10 +107,16 @@ class NetworkTestBase(unittest.TestCase):
                                'veth', 'peer', 'name', 'veth42'])
         klass.dev_e_ap = 'veth42'
         klass.dev_e_client = 'eth42'
+        out = subprocess.check_output(['ip', '-br', 'link', 'show', 'dev', 'eth42'],
+                                      universal_newlines=True)
+        klass.dev_e_client_mac = out.split()[2]
         subprocess.check_call(['ip', 'link', 'add', 'name', 'eth43', 'type',
                                'veth', 'peer', 'name', 'veth43'])
         klass.dev_e2_ap = 'veth43'
         klass.dev_e2_client = 'eth43'
+        out = subprocess.check_output(['ip', '-br', 'link', 'show', 'dev', 'eth43'],
+                                      universal_newlines=True)
+        klass.dev_e2_client_mac = out.split()[2]
 
         # create virtual wlan devs
         before_wlan = set([c for c in os.listdir('/sys/class/net') if c.startswith('wlan')])
@@ -391,6 +397,52 @@ class _CommonTests:
         for i in [self.dev_e_client, self.dev_e2_client, 'mybr']:
             self.assertRegex(out, '%s\s+(ethernet|bridge)\s+%s' % (i, expected_state))
 
+    def test_eth_mtu(self):
+        self.setup_eth(None)
+        self.start_dnsmasq(None, self.dev_e2_ap)
+        with open(self.config, 'w') as f:
+            f.write('''network:
+  renderer: %(r)s
+  ethernets:
+    %(ec)s:
+      dhcp4: yes
+    enmtus:
+      match: {name: %(e2c)s}
+      mtu: 1492
+      dhcp4: yes''' % {'r': self.backend, 'ec': self.dev_e_client, 'e2c': self.dev_e2_client})
+        self.generate_and_settle()
+        self.assert_iface_up(self.dev_e_client,
+                             ['inet 192.168.5.[0-9]+/24'],
+                             ['master'])
+        self.assert_iface_up(self.dev_e2_client,
+                             ['inet 192.168.6.[0-9]+/24'])
+        out = subprocess.check_output(['ip', 'a', 'show', self.dev_e2_client],
+                                      universal_newlines=True)
+        self.assertTrue('mtu 1492' in out, "checking MTU, should be 1492")
+
+    def test_eth_mac(self):
+        self.setup_eth(None)
+        self.start_dnsmasq(None, self.dev_e2_ap)
+        with open(self.config, 'w') as f:
+            f.write('''network:
+  renderer: %(r)s
+  ethernets:
+    %(ec)s:
+      dhcp4: yes
+    enmac:
+      match: {name: %(e2c)s}
+      macaddress: 00:01:02:03:04:05
+      dhcp4: yes''' % {'r': self.backend, 'ec': self.dev_e_client, 'e2c': self.dev_e2_client})
+        self.generate_and_settle()
+        self.assert_iface_up(self.dev_e2_client,
+                             ['inet 192.168.6.[0-9]+/24', '00:01:02:03:04:05'],
+                             ['master'])
+        out = subprocess.check_output(['ip', 'link', 'show', self.dev_e2_client],
+                                      universal_newlines=True)
+        self.assertTrue('ether 00:01:02:03:04:05' in out)
+        subprocess.check_call(['ip', 'link', 'set', self.dev_e2_client,
+                               'address', self.dev_e2_client_mac])
+
     def test_bridge_path_cost(self):
         self.setup_eth(None)
         self.start_dnsmasq(None, self.dev_e2_ap)
@@ -424,6 +476,7 @@ class _CommonTests:
         with open('/sys/class/net/mybr/brif/%s/path_cost' % self.dev_e2_client) as f:
             self.assertEqual(f.read().strip(), '50')
 
+    @unittest.skip("SKIP on 16.10: systemd does not support bridge Priority yet")
     def test_bridge_priority(self):
         self.setup_eth(None)
         self.start_dnsmasq(None, self.dev_e2_ap)
@@ -456,6 +509,7 @@ class _CommonTests:
         with open('/sys/class/net/mybr/bridge/priority') as f:
             self.assertEqual(f.read().strip(), '8192')
 
+    @unittest.skip("SKIP on 16.10: systemd does not support bridge AgeingTime yet")
     def test_bridge_ageing_time(self):
         self.setup_eth(None)
         self.start_dnsmasq(None, self.dev_e2_ap)
@@ -942,31 +996,6 @@ class _CommonTests:
         self.assertIn(b'metric 99',  # check metric from static route
                       subprocess.check_output(['ip', 'route', 'show', '10.10.10.0/24']))
 
-    def test_routes_v6(self):
-        self.setup_eth(None)
-        with open(self.config, 'w') as f:
-            f.write('''network:
-  renderer: %(r)s
-  ethernets:
-    %(ec)s:
-      addresses: ["9876:BBBB::11/70"]
-      gateway6: "9876:BBBB::1"
-      routes:
-          - to: 2001:f00f:f00f::1/64
-            via: 9876:BBBB::5
-            metric: 799''' % {'r': self.backend, 'ec': self.dev_e_client})
-        self.generate_and_settle()
-        self.assert_iface_up(self.dev_e_client,
-                             ['inet6 9876:bbbb::11/70'])
-        self.assertNotIn(b'default',
-                         subprocess.check_output(['ip', 'route', 'show', 'dev', self.dev_e_client]))
-        self.assertIn(b'default via 9876:bbbb::1',
-                      subprocess.check_output(['ip', '-6', 'route', 'show', 'dev', self.dev_e_client]))
-        self.assertIn(b'2001:f00f:f00f::/64 via 9876:bbbb::5',
-                      subprocess.check_output(['ip', '-6', 'route', 'show', 'dev', self.dev_e_client]))
-        self.assertIn(b'metric 799',
-                      subprocess.check_output(['ip', '-6', 'route', 'show', '2001:f00f:f00f::/64']))
-
     def test_manual_addresses(self):
         self.setup_eth(None)
         with open(self.config, 'w') as f:
@@ -1114,6 +1143,27 @@ class _CommonTests:
         self.assertIn(b'default via 192.168.5.1',  # from DHCP
                       subprocess.check_output(['ip', 'route', 'show', 'dev', 'nptesttwo']))
 
+    def test_vlan_mac_address(self):
+        self.setup_eth(None)
+        self.addCleanup(subprocess.call, ['ip', 'link', 'delete', 'myvlan'], stderr=subprocess.DEVNULL)
+        with open(self.config, 'w') as f:
+            f.write('''network:
+  renderer: %(r)s
+  ethernets:
+    ethbn:
+      match: {name: %(ec)s}
+    %(e2c)s: {}
+  vlans:
+    myvlan:
+      id: 101
+      link: ethbn
+      macaddress: aa:bb:cc:dd:ee:22
+        ''' % {'r': self.backend, 'ec': self.dev_e_client, 'e2c': self.dev_e2_client})
+        self.generate_and_settle()
+        self.assert_iface_up('myvlan', ['myvlan@' + self.dev_e_client])
+        with open('/sys/class/net/myvlan/address') as f:
+            self.assertEqual(f.read().strip(), 'aa:bb:cc:dd:ee:22')
+
     def test_wifi_ipv4_open(self):
         self.setup_ap('hw_mode=b\nchannel=1\nssid=fake net', None)
 
@@ -1279,6 +1329,56 @@ wpa_passphrase=12345678
 class TestNetworkd(NetworkTestBase, _CommonTests):
     backend = 'networkd'
 
+    def test_bond_mac(self):
+        self.setup_eth(None)
+        self.addCleanup(subprocess.call, ['ip', 'link', 'delete', 'mybond'], stderr=subprocess.DEVNULL)
+        with open(self.config, 'w') as f:
+            f.write('''network:
+  renderer: %(r)s
+  ethernets:
+    ethbn:
+      match:
+        name: %(ec)s
+        macaddress: %(ec_mac)s
+  bonds:
+    mybond:
+      interfaces: [ethbn]
+      macaddress: 00:01:02:03:04:05
+      dhcp4: yes''' % {'r': self.backend,
+                       'ec': self.dev_e_client,
+                       'e2c': self.dev_e2_client,
+                       'ec_mac': self.dev_e_client_mac})
+        self.generate_and_settle()
+        self.assert_iface_up(self.dev_e_client,
+                             ['master mybond'])
+        self.assert_iface_up('mybond',
+                             ['inet 192.168.5.[0-9]+/24', '00:01:02:03:04:05'])
+
+    def test_bridge_mac(self):
+        self.setup_eth(None)
+        self.start_dnsmasq(None, self.dev_e2_ap)
+        with open(self.config, 'w') as f:
+            f.write('''network:
+  renderer: %(r)s
+  ethernets:
+    ethbr:
+      match:
+        name: %(ec)s
+        macaddress: %(ec_mac)s
+  bridges:
+    br0:
+      interfaces: [ethbr]
+      macaddress: 00:01:02:03:04:05
+      dhcp4: yes''' % {'r': self.backend,
+                       'ec': self.dev_e_client,
+                       'e2c': self.dev_e2_client,
+                       'ec_mac': self.dev_e_client_mac})
+        self.generate_and_settle()
+        self.assert_iface_up(self.dev_e_client,
+                             ['master br0'], ['inet'])
+        self.assert_iface_up('br0',
+                             ['inet 192.168.5.[0-9]+/24', '00:01:02:03:04:05'])
+
     def test_bond_down_delay(self):
         self.setup_eth(None)
         self.addCleanup(subprocess.call, ['ip', 'link', 'delete', 'mybond'], stderr=subprocess.DEVNULL)
@@ -1460,10 +1560,43 @@ class TestNetworkd(NetworkTestBase, _CommonTests):
         with open('/sys/class/net/mybond/bonding/arp_validate') as f:
             self.assertEqual(f.read().strip(), 'all 3')
 
+    def test_routes_v6(self):
+        self.setup_eth(None)
+        with open(self.config, 'w') as f:
+            f.write('''network:
+  renderer: %(r)s
+  ethernets:
+    %(ec)s:
+      addresses: ["9876:BBBB::11/70"]
+      gateway6: "9876:BBBB::1"
+      routes:
+          - to: 2001:f00f:f00f::1/64
+            via: 9876:BBBB::5
+            metric: 799''' % {'r': self.backend, 'ec': self.dev_e_client})
+        self.generate_and_settle()
+        self.assert_iface_up(self.dev_e_client,
+                             ['inet6 9876:bbbb::11/70'])
+        self.assertNotIn(b'default',
+                         subprocess.check_output(['ip', 'route', 'show', 'dev', self.dev_e_client]))
+        self.assertIn(b'default via 9876:bbbb::1',
+                      subprocess.check_output(['ip', '-6', 'route', 'show', 'dev', self.dev_e_client]))
+        self.assertIn(b'2001:f00f:f00f::/64 via 9876:bbbb::5',
+                      subprocess.check_output(['ip', '-6', 'route', 'show', 'dev', self.dev_e_client]))
+        self.assertIn(b'metric 799',
+                      subprocess.check_output(['ip', '-6', 'route', 'show', '2001:f00f:f00f::/64']))
+
 
 class TestNetworkManager(NetworkTestBase, _CommonTests):
     backend = 'NetworkManager'
 
+    @unittest.skip("NetworkManager does not support setting MAC for a bond")
+    def test_bond_mac(self):
+        pass
+
+    @unittest.skip("NetworkManager does not support setting MAC for a bridge")
+    def test_bridge_mac(self):
+        pass
+
     def test_wifi_ap_open(self):
         # we use dev_w_client and dev_w_ap in switched roles here, to keep the
         # existing device blacklisting in NM; i. e. dev_w_client is the
@@ -1620,6 +1753,7 @@ class TestNetworkManager(NetworkTestBase, _CommonTests):
         with open('/sys/class/net/mybond/bonding/arp_ip_target') as f:
             self.assertEqual(f.read().strip(), '192.168.5.1')
 
+    @unittest.skip("SKIP 16.10: broken on 16.10 versions of linux/NetworkManager")
     def test_bond_arp_all_targets(self):
         self.setup_eth(None)
         self.start_dnsmasq(None, self.dev_e2_ap)
@@ -1683,5 +1817,9 @@ class TestNetworkManager(NetworkTestBase, _CommonTests):
         with open('/sys/class/net/mybond/bonding/lp_interval') as f:
             self.assertEqual(f.read().strip(), '15')
 
+    @unittest.skip("SKIP 16.10: broken on 16.10 versions of linux/NetworkManager")
+    def test_routes_v6(self):
+        pass
+
 
 unittest.main(testRunner=unittest.TextTestRunner(stream=sys.stdout, verbosity=2))
